/*
** file: main-maxh.cpp
**
** max Heap (binary tree on array)
*/

// Defina neste ficheiro:
//   A entrada/saída de dados
//   As instâncias da classe da estrutura de dados
//   A implementação dos comandos através dos métodos da classe
//   Código auxiliar
//   Não utilize variáveis globais!

#include <iostream>
#include <sstream>
#include <string>
#include "maxh.h"
using namespace std;


int main()
{
    const int initialCapacity = 15;  // Define a capacidade inicial do heap
    
    // Cria uma instância do max heap com capacidade definida por initialCapacity
    IMAXH maxheap(initialCapacity);
    
    int currentCapacity = initialCapacity; // Inicializa a capacidade atual com a capacidade inicial do heap
    
    /* Sugere-se que seja lida uma linha de cada vez para uma string e analisado
    * o primeiro carater para ver se é um '#', caso em que a linha é um comentário. 
    * Caso não seja comentário, criar uma string stream e usar o operador extrator >>
    * para obter o nome do comando. Terá sucesso se não for uma linha em branco. 
    */

    // Variáveis para armazenar cada linha lida e o comando extraído
    string linha, comando;
        
    while (getline(cin, linha)) {   // Lê uma linha inteira de cada vez
        if (linha.empty() || linha[0] == '#') {
            continue;               // Ignora linhas vazias e comentários
        }

        stringstream ss(linha);     // Usa stringstream para processar a linha
        ss >> comando;              // Extrai o nome do comando

        // Processar o comando extraído
        if (comando == "insert") {
            int item;
            while (ss >> item) {    // Extrair todos os itens da linha para inserir no início
                if (maxheap.isFull()) {
                    cout << "Comando insert: Heap cheio!\n";
                break;              // Interrompe a leitura e inserção de mais itens
                }
                maxheap.insert(item);   // Insere item no heap
            }
        } else if (comando == "print_max") {    // imprime o maior item no heap
            maxheap.print_max();
        } else if (comando == "print") {        // imprime todos os itens do heap
            maxheap.print();
        } else if (comando == "dim") {          // imprime o número atual de itens no heap
            maxheap.dim();
        } else if (comando == "dim_max") {      // imprime a capacidade máxima do heap
            maxheap.dim_max();
        } else if (comando == "clear") {        // remove todos os itens do heap
            maxheap.clear();
        } else if (comando == "delete") {       // remove o maior item do heap
            maxheap.delete_max();
        } else if (comando == "heapify_up") {   // organiza os itens dados num max_heap
            // Aloca dinamicamente o array temporário com a capacidade atual do heap
            int* temp = new int[currentCapacity];
            int count = 0;
            int item;
            while (ss >> item && count < currentCapacity) {
                temp[count++] = item; // Armazena cada item no array temporário
            }
            if (count > 0) {
                maxheap.heapify_up(temp, count); // Se houver itens no array, organiza-os num max_heap
            }
            delete[] temp; // Liberta a memória alocada dinamicamente
        } else if (comando == "redim_max") { // redimensiona a capacidade máxima do heap
            int newCapacity;
            if (ss >> newCapacity) {
                currentCapacity = maxheap.redim_max(newCapacity);
            }
        }
    }
    return 0;
}

// EOF
