/*
** file: maxh.h
**
** max Heap (binary tree on array)
*/

// Defina:
//   Em maxh.h as classes da estrutura de dados
//   Em maxh.cpp a implementação dos métodos das classes da estrutura de dados
// Não altere o nome das classes nem dos atributos obrigatórios!

#ifndef MAXH_H
#define MAXH_H

// definir int max heap em array
class IMAXH {
private:
   // atributos obrigatórios
   int *v,     // vetor com nós
       n,      // num. nós usados  (dimensão atual do vetor)
       nv;     // dim max do vetor (capacidade)

    // Métodos privados para manipulação de índices
    int getParentIndex(int childIndex);
    int getLeftChildIndex(int parentIndex);
    int getRightChildIndex(int parentIndex);
    void shiftUp(int index);    // Para manter a propriedade do heap após inserção
    void shiftDown(int index);  // Para manter a propriedade do heap após remoção

public:
    // construtores
    IMAXH(int nmax=15);  // cria heap vazio c/ capacidade nmax nós
    ~IMAXH();            // Destrutor: liberta memória alocada
    // outros atributos e métodos (protótipos) livres

    // Operações do heap
    void insert(int item);      // Insere um novo item no heap e restaura a propriedade de heap se necessário
    void print_max() const;     // Mostra o maior item no heap
    void print() const;         // Mostra todos os itens no heap em formato de árvore
    void dim() const;           // Mostra o número atual de itens no heap
    void dim_max() const;       // Mostra a capacidade máxima do heap
    void clear();               // Limpa o heap, removendo todos os elementos
    void delete_max();          // Remove o maior elemento do heap e restaura a propriedade de heap
    void heapify_up(const int* items, int size);    // Rearranja os elementos dados para formar um heap
    int redim_max(int newCapacity);    // Redimensiona o heap para uma nova capacidade máxima
    
    // Métodos auxiliares
    bool isFull() const { return n == nv; }

};

#endif // MAXH_H
// EOF